# Efficient Content-Based Sparse Attention with Routing Transformers

This directory contains code for ICLR submission titled "Efficient Content-Based
Sparse Attention with Routing Transformers".

Example command to train a smaller model on `wikitext-103`. Please note that the
reported results are with bigger models and the hyper params are explained in
the paper.

```
python3 -m routing_transformers_iclr20.t2t_trainer \
  --problem=languagemodel_wikitext103 \
  --hparams_set=wikitext103_local_cluster_small \
  --model=sparse_transformer \
  --data_dir=<data_dir> \
  --output_dir=<output_dir> \
  --train_steps=500000
```

This code relies on the Tensor2Tensor framework. See
https://github.com/tensorflow/tensor2tensor for more information.

## Explanation of hyperparameters

For local attention the main hyperparameters are `query_shape`, `memory_flange`
and `local_num_heads`. The hparams `query_shape` and `memory flange` represent
the size of the local block used for local attention, for e.g. if the total
block size of local attention is 512, then `query_shape` and `memory_flange`
should be set to (256,) each. The hparam `local_num_heads` represents the number
of local attention heads. An example hparam setting can be found in the
`sparse_transformer.py` file under the name `wikitext103_local4k`.

For routing attention the main hyperparameters are `sparsity_cluster_z_size`,
`sparsity_cluster_attention_window` and `sparsity_cluster_num_heads`. The
hparam`sparsity_cluster_z_size` represents the number of clusters in log_2
scale. The hparam `sparsity_cluster_attention_window` represents the average
size of a cluster. So for example, if the total sequence length is 8192, and
`sparsity_cluster_attention_window` is set to 512, a good choice for
`sparsity_cluster_z_size` could be 4 (8192/512=16=2^4). In practice, we also
divide the total attention head budget equally between local and routing
attention, so `sparsity_cluster_num_heads` is set to be equal to
`local_num_heads`. An example hparam setting can be found in the
`sparse_transformer.py` file under the name `wikitext103_local_cluster4k`.

The other main hparam to think about would be `num_decoder_layers`. For local
attention you want this to be very high (as much as fits in your memory).
However, for Routing Transformers the number of decoder layers needed is lower
(since each attention layer is more parameter heavy). Usually, set it to be half
the number of layers used in local.

Other hparams to take care of are `max_relative_position`, which denotes the max
distance considered for relative attention. This should be bigger than
`query_shape + memory_flange` and `sparsity_cluster_attention_window`.
